use crate::util::surrealdb::{client::sort_participants, client::Time, schemas};
use leptos::*;

/// Renders the home page of your application.
#[component]
pub fn Participants() -> impl IntoView {
    let participants_context = use_context::<schemas::ParticipantsContext>().unwrap();

    let name = create_rw_signal("".to_string());

    let participants_sorted =
        create_memo(move |_| sort_participants(participants_context.get(), name.get()));

    view! {
        <input type="search"
            class="participants-search"
            placeholder="Search"
            autocomplete="off"
            autofocus=true
            on:input=move |ev| {
                name.set(event_target_value(&ev));
            }
            prop:value=name
        />
        <table class="participants-table">
            <tr>
                <th>"Naam"</th>
                <th>"Groep"</th>
                <th>"Lifesaver"</th>
                <th>"Hindernis"</th>
                <th>"Popduiken"</th>
            </tr>
            <For
                each=move || participants_sorted.get()
                key=|state| state.id.clone()
                let:child
            >
                <tr>
                    <td>{ move || child.value.get().name }</td>
                    <td>{ move || child.value.get().group }</td>
                    { move || match child.value.get().events {
                            Some(events) => view! {
                                <td>{ Time::from_milliseconds_to_string(events.lifesaver.unwrap_or(0)) }</td>
                                <td>{ Time::from_milliseconds_to_string(events.hindernis.unwrap_or(0)) }</td>
                                <td>{ Time::from_milliseconds_to_string(events.popduiken.unwrap_or(0)) }</td>
                            },
                            None => view! {
                                <td>"0"</td>
                                <td>"0"</td>
                                <td>"0"</td>
                            }
                        }
                    }
                </tr>
            </For>
        </table>
    }
}
