use crate::util::surrealdb::schemas;
use leptos::*;
use leptos_use::*;

use super::ParticipantsAction;

pub fn init_websocket() {
    let UseWebsocketReturn {
        ready_state,
        message,
        ..
    } = use_websocket("ws://localhost:3000/ws");

    provide_context(ready_state);
    let participants_context = use_context::<schemas::ParticipantsContext>().unwrap();
    let owner = Owner::current().unwrap();

    create_effect(move |_| {
        if let Some(m) = message.get() {
            with_owner(owner, || handle_message(&participants_context, &m));
        }
    });
}

fn handle_message(&participants_context: &schemas::ParticipantsContext, message: &str) {
    let action: ParticipantsAction = match serde_json::from_str(message) {
        Ok(res) => res,
        Err(_err) => {
            return;
        }
    };

    logging::log!("Recieved action: {:?}", action);

    match action {
        ParticipantsAction::Add { participant } => participants_context.update(|participants| {
            participants.push(schemas::ParticipantSignal {
                id: participant.id.clone(),
                value: create_rw_signal(participant),
            })
        }),
        ParticipantsAction::Remove { id } => participants_context
            .update(|participants| participants.retain(|participant| participant.id != id)),
        ParticipantsAction::Replace { participant } => {
            let participants = participants_context.get();
            for participant_signal in participants {
                if participant_signal.id == participant.id {
                    participant_signal.value.set(participant);
                    break;
                }
            }
        }
    }
}
